"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.dataURItoBlob = dataURItoBlob;
exports.canvasDataURL = canvasDataURL;
exports.setCookies = setCookies;
exports.setCookie = setCookie;
exports.getCookie = getCookie;
exports.deepClone = deepClone;
exports.getFileURL = getFileURL;
exports.propsMapper = propsMapper;
exports.isUnValid = isUnValid;
exports.timeoutPromise = timeoutPromise;
exports.parseJson = parseJson;
exports.isFunction = isFunction;
exports.json2param = json2param;
exports.param2json = param2json;
exports.renderTemplate = renderTemplate;
exports.promiseAll = promiseAll;

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function dataURItoBlob(dataURI) {
  var byteString = atob(dataURI.split(',')[1]);
  var mimeString = dataURI.split(',')[0].split(':')[1].split(';')[0];
  var ab = new ArrayBuffer(byteString.length);
  var ia = new Uint8Array(ab);

  for (var i = 0; i < byteString.length; i++) {
    ia[i] = byteString.charCodeAt(i);
  }

  return new Blob([ab], {
    type: mimeString
  });
} // 图片压缩


function canvasDataURL(path) {
  var quality = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0.6;
  var img = new Image();
  return new Promise(function (resolve, reject) {
    img.src = path;

    img.onload = function () {
      // 默认按比例压缩
      var w = img.width,
          h = img.height,
          scale = w / h;
      w = Math.min(2000, w);
      h = w / scale; //生成canvas

      var canvas = document.createElement('canvas');
      var ctx = canvas.getContext('2d'); // 创建属性节点

      var anw = document.createAttribute("width");
      anw.nodeValue = w;
      var anh = document.createAttribute("height");
      anh.nodeValue = h;
      canvas.setAttributeNode(anw);
      canvas.setAttributeNode(anh);
      ctx.drawImage(img, 0, 0, w, h); // quality值越小，所绘制出的图像越模糊

      var base64 = canvas.toDataURL('image/jpeg', quality); // 回调函数返回base64的值

      resolve(base64);
    };

    img.onerror = function (err) {
      reject(err);
    };
  });
}

function setCookies(obj, cookieExpiresTime) {
  for (var key in obj) {
    var v = obj[key];

    if (obj.hasOwnProperty(key) && v) {
      setCookie(key, v, cookieExpiresTime);
    }
  }
}

function setCookie(name, value) {
  var cookieExpiresTime = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 24 * 60 * 60 * 30;
  var exp = new Date();
  exp.setTime(Date.now() + cookieExpiresTime);
  document.cookie = name + "=" + escape(value) + ";expires=" + exp.toGMTString();
}

function getCookie(name) {
  var arr;
  var reg = new RegExp("(^| )" + name + "=([^;]*)(;|$)");

  if (arr = document.cookie.match(reg)) {
    return unescape(arr[2]);
  }

  return null;
}

function deepClone(obj) {
  return JSON.parse(JSON.stringify(obj));
}

function getFileURL(file) {
  var getUrl = null;

  if (window.createObjectURL != undefined) {
    // basic
    getUrl = window.createObjectURL(file);
  } else if (window.URL != undefined) {
    // mozilla(firefox)
    getUrl = window.URL.createObjectURL(file);
  } else if (window.webkitURL != undefined) {
    // webkit or chrome
    getUrl = window.webkitURL.createObjectURL(file);
  }

  return getUrl;
} // 获取对象a.b.ckey的值


function propsMapper(props, keyStr) {
  var defaultValue = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
  var newKeyName = arguments.length > 3 ? arguments[3] : undefined;
  var keys = keyStr.split('.');
  var keyValue = objectKeysValue(props, keys, defaultValue);
  return _defineProperty({}, newKeyName || keys[keys.length - 1], keyValue);
} // 获取对象[key]数组的值


function objectKeysValue(object, keys) {
  var defaultValue = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

  if (isUnValid(object)) {
    return defaultValue;
  }

  if (keys.length > 1) {
    return objectKeysValue(object[keys[0]], keys.slice(1), defaultValue);
  } else if (keys.length === 1) {
    if (!object) {
      return defaultValue;
    }

    var v = object[keys[0]];
    return isUnValid(v) ? defaultValue : v;
  } else {
    return {};
  }
} // 是否为无效值


function isUnValid(value) {
  return !value && value !== false && value !== 0;
} // 定时任务返回Promise


function timeoutPromise(fun) {
  var timeout = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1000;

  if (!isFunction(fun)) {
    return Promise.resolve();
  }

  return new Promise(function (resolve, reject) {
    setTimeout(function () {
      try {
        resolve(fun());
      } catch (err) {
        reject(err);
      }
    }, timeout);
  });
} // 解析成json


function parseJson(text) {
  try {
    return JSON.parse(text);
  } catch (_unused) {
    return text;
  }
} // 解析成json


function isFunction(fun) {
  return typeof fun === 'function';
}
/**
 * 将json对象转换成键值对字符串
 * @param o json对象
 * @returns {string}
 */


function json2param(o) {
  var p = [];

  for (var key in o) {
    if (o.hasOwnProperty(key)) {
      var value = o[key];

      if (value == null || value === undefined || Number.isNaN(value) || value === Infinity || value === -Infinity) {
        value = '';
      }

      p.push("".concat(key, "=").concat(encodeURIComponent(value)));
    }
  }

  return p.join('&');
}
/**
 * 将键值对字符串转换成json对象
 * @param str 键值对字符串
 * @returns {object}
 */


function param2json(str) {
  if (typeof str !== 'string') {
    return {};
  }

  var i1 = str.indexOf('?');
  var i2 = str.indexOf('#');

  if (i2 < 0) {
    i2 = str.length;
  }

  if (i2 > i1) {
    str = str.substring(i1 + 1, i2);
  }

  var reg = /([a-z0-9_]+)=([^&#]+)/ig;
  var m = reg.exec(str);
  var ret = {};

  while (m) {
    ret[m[1]] = decodeURIComponent(m[2]);
    m = reg.exec(str);
  }

  return ret;
}
/**
 * 简单模板引擎
 * @param template 模板字符串
 * @param context 上下文数据
 * @returns {string}
 */


function renderTemplate(template, context) {
  var reg = /\{\{(.*?)\}\}/g;

  var repl = function repl(match, key) {
    return context[key.trim()];
  };

  return template.replace(reg, repl);
}
/**
 * 数组异步回调
 * @param arr 包含Promise的数组或者数据，如果为数据，则必须传operate操作函数
 * @param operate 要执行的操作，返回Promise对象（主要是为了真正的同步执行数组中的Promise）
 * @returns {Promise<any>}
 */


function promiseAll(arr, operate) {
  var resultArr = [];

  function _aysncArr(cb) {
    var a = arr.shift();

    if (a) {
      var promise = a;

      if (isFunction(operate)) {
        promise = operate(a);
      }

      promise.then(function (r) {
        resultArr.push(r);

        _aysncArr(cb);
      })["catch"](function (err) {
        cb(err, resultArr);
      });
    } else {
      cb(null, resultArr);
    }
  }

  return new Promise(function (resolve, reject) {
    _aysncArr(function (err, res) {
      if (err) {
        reject(err);
      } else {
        resolve(res);
      }
    });
  });
}